"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculatePricingInformation = exports.priceFormatter = void 0;
const geo_shipping_prices_1 = require("@uppfirst/geo-shipping-prices");
const decimal_js_1 = __importDefault(require("decimal.js"));
const utils_1 = require("./utils");
Object.defineProperty(exports, "priceFormatter", { enumerable: true, get: function () { return utils_1.priceFormatter; } });
const currencyCodeToSymbolMap = {
    usd: '$',
    eur: '€',
    gbp: '£',
    jpy: '¥',
};
const calculateShippingPrices = ({ product, stateCode, countryCode, traitCost, decimals, }) => {
    var _a, _b;
    const shipping = product.shipping;
    if (!shipping || !countryCode) {
        const zero = product.price instanceof decimal_js_1.default ? new decimal_js_1.default(0) : 0;
        return {
            taxCost: zero,
            taxPercent: zero,
            shippingCost: zero,
        };
    }
    const shippingCalculator = new geo_shipping_prices_1.Shipping(shipping);
    const prices = stateCode && countryCode === 'US'
        ? shippingCalculator.state(stateCode)
        : shippingCalculator.country(countryCode);
    const taxPercent = (0, utils_1.ensureDecimal)(((_a = prices.prices) === null || _a === void 0 ? void 0 : _a.tax) || 0);
    const shippingCost = (0, utils_1.ensureDecimal)(((_b = prices.prices) === null || _b === void 0 ? void 0 : _b.price) || 0);
    const price = (0, utils_1.ensureDecimal)(product.price);
    const calculatedShippingCost = (0, utils_1.multiplyByDecimals)(shippingCost, decimals);
    const taxableCost = price.plus(calculatedShippingCost).plus(traitCost);
    // Determine if we should return Decimal based on input type
    const shouldReturnDecimal = product.price instanceof decimal_js_1.default;
    const calculatedTaxCost = taxPercent
        .dividedBy(100)
        .times(taxableCost)
        .round();
    return {
        taxCost: shouldReturnDecimal
            ? calculatedTaxCost
            : calculatedTaxCost.toNumber(),
        taxPercent: shouldReturnDecimal ? taxPercent : taxPercent.toNumber(),
        shippingCost: calculatedShippingCost,
    };
};
const calculatePricingInformation = ({ product, countryCode, stateCode, combination, usedRewards = [], }, opts) => {
    const { returnSmallestUnit } = opts || {};
    // Convert all inputs to Decimal for calculations
    const startAmountPercent = (0, utils_1.ensureDecimal)(product.startAmountPercent).dividedBy(100);
    const rewardShare = (0, utils_1.ensureDecimal)(product.rewardShare).dividedBy(100);
    const claimPercent = new decimal_js_1.default(1).minus(startAmountPercent);
    const price = (0, utils_1.ensureDecimal)(product.price);
    const traitCost = (0, utils_1.ensureDecimal)((combination === null || combination === void 0 ? void 0 : combination.price) || 0);
    const baseReserve = price.times(startAmountPercent);
    const baseClaim = price.times(claimPercent);
    const gainedRewardsReserve = rewardShare.times(baseReserve);
    const gainedRewardsClaim = rewardShare.times(baseClaim);
    const totalBase = price;
    const decimals = (0, utils_1.ensureDecimal)(product.currency.decimals);
    const { shippingCost, taxCost, taxPercent } = calculateShippingPrices({
        product,
        countryCode,
        stateCode,
        traitCost,
        decimals,
    });
    const shippingCostDecimal = (0, utils_1.ensureDecimal)(shippingCost);
    const taxCostDecimal = (0, utils_1.ensureDecimal)(taxCost);
    const taxPercentDecimal = (0, utils_1.ensureDecimal)(taxPercent);
    const usedRewardsTotal = usedRewards.reduce((acc, reward) => acc.plus((0, utils_1.ensureDecimal)(reward.worth)), new decimal_js_1.default(0));
    return {
        currency: {
            code: product.currency.symbol.toLowerCase(),
            symbol: currencyCodeToSymbolMap[product.currency.symbol.toLocaleLowerCase()],
            decimals: (0, utils_1.convertBack)(decimals, 0),
        },
        startAmountPercent: startAmountPercent.times(100).toNumber(),
        usedRewardsTotal: (0, utils_1.divideConvertBackSmallestUnit)(usedRewardsTotal, decimals, returnSmallestUnit),
        reserve: {
            base: (0, utils_1.divideConvertBackSmallestUnit)(baseReserve, decimals, returnSmallestUnit),
            traitCost: (0, utils_1.divideConvertBackSmallestUnit)(traitCost, decimals, returnSmallestUnit),
            gainedRewardsAmount: (0, utils_1.divideConvertBackSmallestUnit)(gainedRewardsReserve, decimals, returnSmallestUnit),
            taxPercent: (0, utils_1.convertBack)(taxPercentDecimal, decimals),
            shippingCost: (0, utils_1.divideConvertBackSmallestUnit)(shippingCostDecimal, decimals, returnSmallestUnit),
            taxCost: (0, utils_1.divideConvertBackSmallestUnit)(taxCostDecimal, decimals, returnSmallestUnit),
            total: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(baseReserve
                .plus(traitCost)
                .plus(taxCostDecimal)
                .plus(shippingCostDecimal)
                // TODO: only reserved rewards should be subtracted here
                .minus(usedRewardsTotal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
        claim: {
            base: (0, utils_1.divideConvertBackSmallestUnit)(baseClaim, decimals, returnSmallestUnit),
            gainedRewardsAmount: (0, utils_1.divideConvertBackSmallestUnit)(gainedRewardsClaim, decimals, returnSmallestUnit),
            total: (0, utils_1.divideConvertBackSmallestUnit)(
            // TODO: only claimed rewards should be subtracted here
            decimal_js_1.default.max(baseClaim.minus(usedRewardsTotal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
        total: {
            base: (0, utils_1.divideConvertBackSmallestUnit)(totalBase, decimals, returnSmallestUnit),
            gainedRewardsAmount: (0, utils_1.divideConvertBackSmallestUnit)(gainedRewardsReserve.plus(gainedRewardsClaim), decimals, returnSmallestUnit),
            total: (0, utils_1.divideConvertBackSmallestUnit)(decimal_js_1.default.max(totalBase
                .plus(traitCost)
                .plus(taxCostDecimal)
                .plus(shippingCostDecimal)
                .minus(usedRewardsTotal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
    };
};
exports.calculatePricingInformation = calculatePricingInformation;
