"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.calculatePricingInformation = void 0;
const geo_shipping_prices_1 = require("@uppfirst/geo-shipping-prices");
const decimal_js_1 = __importDefault(require("decimal.js"));
const currencyCodeToSymbolMap = {
    usd: '$',
    eur: '€',
    gbp: '£',
    jpy: '¥',
};
const calculateShippingPrices = ({ product, stateCode, countryCode, traitCost, decimals, }) => {
    var _a, _b;
    const shipping = product.shipping;
    if (!shipping || !countryCode) {
        const zero = product.price instanceof decimal_js_1.default ? new decimal_js_1.default(0) : 0;
        return {
            taxCost: zero,
            taxPercent: zero,
            shippingCost: zero,
        };
    }
    const shippingCalculator = new geo_shipping_prices_1.Shipping(shipping);
    const prices = stateCode && countryCode === 'US'
        ? shippingCalculator.state(stateCode)
        : shippingCalculator.country(countryCode);
    const taxPercent = ensureDecimal(((_a = prices.prices) === null || _a === void 0 ? void 0 : _a.tax) || 0);
    const shippingCost = ensureDecimal(((_b = prices.prices) === null || _b === void 0 ? void 0 : _b.price) || 0);
    const price = ensureDecimal(product.price);
    const calculatedShippingCost = multiplyByDecimals(shippingCost, decimals);
    const taxableCost = price.plus(calculatedShippingCost).plus(traitCost);
    // Determine if we should return Decimal based on input type
    const shouldReturnDecimal = product.price instanceof decimal_js_1.default;
    const calculatedTaxCost = taxPercent
        .dividedBy(100)
        .times(taxableCost)
        .round();
    return {
        taxCost: shouldReturnDecimal
            ? calculatedTaxCost
            : calculatedTaxCost.toNumber(),
        taxPercent: shouldReturnDecimal ? taxPercent : taxPercent.toNumber(),
        shippingCost: calculatedShippingCost,
    };
};
// Helper functions to handle both number and Decimal inputs
const ensureDecimal = (value) => {
    return value instanceof decimal_js_1.default ? value : new decimal_js_1.default(value);
};
const convertBack = (value, decimals) => {
    const decimalPlaces = ensureDecimal(decimals);
    if (decimalPlaces.isZero()) {
        return value.round().toNumber();
    }
    // If any input was Decimal, return Decimal, otherwise return number
    return value.toDecimalPlaces(decimalPlaces.toNumber()).toNumber();
};
const divideByDecimals = (value, decimals) => {
    return value.dividedBy(new decimal_js_1.default(10).pow(ensureDecimal(decimals).toNumber()));
};
const divideConvertBack = (value, decimals) => {
    return convertBack(divideByDecimals(value, decimals), decimals);
};
const multiplyByDecimals = (value, decimals) => {
    return value.times(new decimal_js_1.default(10).pow(decimals));
};
/**
 * Divide a value by decimals and convert back to the original unit, but if `returnSmallestUnit` is true, return the
 * smallest unit instead or main unit.
 * @param value
 * @param decimals
 * @param returnSmallestUnit
 */
const divideConvertBackSmallestUnit = (value, decimals, returnSmallestUnit) => {
    return returnSmallestUnit
        ? convertBack(value, 0)
        : divideConvertBack(value, decimals);
};
const calculatePricingInformation = ({ product, countryCode, stateCode, combination, usedRewards = [], }, opts) => {
    const { returnSmallestUnit } = opts || {};
    // Convert all inputs to Decimal for calculations
    const startAmountPercent = ensureDecimal(product.startAmountPercent).dividedBy(100);
    const rewardShare = ensureDecimal(product.rewardShare).dividedBy(100);
    const claimPercent = new decimal_js_1.default(1).minus(startAmountPercent);
    const price = ensureDecimal(product.price);
    const traitCost = ensureDecimal((combination === null || combination === void 0 ? void 0 : combination.price) || 0);
    const baseReserve = price.times(startAmountPercent);
    const baseClaim = price.times(claimPercent);
    const gainedRewardsReserve = rewardShare.times(baseReserve);
    const gainedRewardsClaim = rewardShare.times(baseClaim);
    const totalBase = price;
    const decimals = ensureDecimal(product.currency.decimals);
    const { shippingCost, taxCost, taxPercent } = calculateShippingPrices({
        product,
        countryCode,
        stateCode,
        traitCost,
        decimals,
    });
    console.log(taxCost, 'sina taxCost');
    const shippingCostDecimal = ensureDecimal(shippingCost);
    const taxCostDecimal = ensureDecimal(taxCost);
    const taxPercentDecimal = ensureDecimal(taxPercent);
    const usedRewardsTotal = usedRewards.reduce((acc, reward) => acc.plus(ensureDecimal(reward.worth)), new decimal_js_1.default(0));
    return {
        currency: {
            code: product.currency.symbol.toLowerCase(),
            symbol: currencyCodeToSymbolMap[product.currency.symbol.toLocaleLowerCase()],
            decimals: convertBack(decimals, 0),
        },
        startAmountPercent: startAmountPercent.times(100).toNumber(),
        usedRewardsTotal: divideConvertBackSmallestUnit(usedRewardsTotal, decimals, returnSmallestUnit),
        reserve: {
            base: divideConvertBackSmallestUnit(baseReserve, decimals, returnSmallestUnit),
            traitCost: divideConvertBackSmallestUnit(traitCost, decimals, returnSmallestUnit),
            gainedRewardsAmount: divideConvertBackSmallestUnit(gainedRewardsReserve, decimals, returnSmallestUnit),
            taxPercent: convertBack(taxPercentDecimal, decimals),
            shippingCost: divideConvertBackSmallestUnit(shippingCostDecimal, decimals, returnSmallestUnit),
            taxCost: divideConvertBackSmallestUnit(taxCostDecimal, decimals, returnSmallestUnit),
            total: divideConvertBackSmallestUnit(decimal_js_1.default.max(baseReserve
                .plus(traitCost)
                .plus(taxCostDecimal)
                .plus(shippingCostDecimal)
                // TODO: only reserved rewards should be subtracted here
                .minus(usedRewardsTotal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
        claim: {
            base: divideConvertBackSmallestUnit(baseClaim, decimals, returnSmallestUnit),
            gainedRewardsAmount: divideConvertBackSmallestUnit(gainedRewardsClaim, decimals, returnSmallestUnit),
            total: divideConvertBackSmallestUnit(
            // TODO: only claimed rewards should be subtracted here
            decimal_js_1.default.max(baseClaim.minus(usedRewardsTotal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
        total: {
            base: divideConvertBackSmallestUnit(totalBase, decimals, returnSmallestUnit),
            gainedRewardsAmount: divideConvertBackSmallestUnit(gainedRewardsReserve.plus(gainedRewardsClaim), decimals, returnSmallestUnit),
            total: divideConvertBackSmallestUnit(decimal_js_1.default.max(totalBase
                .plus(traitCost)
                .plus(taxCostDecimal)
                .plus(shippingCostDecimal)
                .minus(usedRewardsTotal), new decimal_js_1.default(0)), decimals, returnSmallestUnit),
        },
    };
};
exports.calculatePricingInformation = calculatePricingInformation;
